/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file connection.cc
/// \brief Implementation of Connection class

#include "connection.hh"
#include "routines.hh"

#include <iostream>


//////////////////////////////////////
//				PUBLIC				//
//////////////////////////////////////


// Constructor
Connection::Connection( Settings* settings ): engine_( NULL ), 
	btserver_( NULL ), ipserver_( NULL ), settings_( settings ), 
	connecting_( false ), online_( false )
{
	// get data for active connection profile
	Profile profile = {};
	
	if( settings_->getUseDefaultProfile() )
	{
		if( !settings_->getDefaultProfile( profile ) )
		{
			std::cout << "Could not get default profile settings" << std::endl;
		}
	}
	else
	{
		if( !settings_->getActiveProfile( profile ) )
		{
			std::cout << "Could not get active profile settings" << std::endl;
		}
	}
	
	// save profile for future reference
	profile_ = profile;
	
}

// Destructor
Connection::~Connection()
{
	//std::cout << "@connection destructor" << std::endl;
	clearEngine();
	
}

// Establish a connection to mythetomerserver
Status Connection::openConnection()
{
	if( engine_ != NULL && engine_->isConnected() )
	{
		return WASCONNECTED;
	}
	
	// read the profile just in case it has been changed
	Profile profile = {};
	if( !settings_->getActiveProfile( profile ) )
	{
		std::cout << "Could not get profile" << std::endl;
		return FAILED;
	}
	
	profile_ = profile;
	
	if( !initEngine( profile_ ) )
	{
		return FAILED;
	}
	
	// make sure the WLAN connection to access point is open,
	// if not then open if needed
	if( engine_ == ipserver_ )
	{
		
		if( !openWlanConnection() )
		{
			std::cout << "Could not open WLAN connection" << std::endl;
			return FAILED;
		}
	}
	
	if( engine_ != NULL && !engine_->connectServer() )
	{
		return FAILED;
	}
	
	return CONNECTED;

}

// Close the network connection
void Connection::closeConnection()
{
	if( engine_ != NULL )
	{
		engine_->disconnect();
	}
	
	// TODO:
	// close also opened wlan connection??
}

// Change connection profile
bool Connection::changeConnectionProfile( std::string const& name )
{
	if( name != profile_.name )
	{
		Profile profile = {};
	
		if( !settings_->getProfile( name, profile ) )
		{
			std::cout << "Could not get profile" << std::endl;
			return false;
		}
	
		profile_ = profile;
		
		if( !initEngine( profile ) )
		{
			return false;
		}
	}
	
	return true;
}

// Get info whether the connection is currently established
bool Connection::isConnected() const
{
	if( engine_ != NULL )
	{
		return engine_->isConnected();
	}
	
	return false;
}

// Send message to server
bool Connection::sendMsg( std::string msg )
{
	if( engine_ != NULL )
	{
		return engine_->sendMsg( msg );
	}
	
	return false;
}

// Handle dbus event from ConIcConnection
void Connection::handleConnectionEvent( ConIcConnectionEvent* event )
{
	switch( con_ic_connection_event_get_status( event ) )
	{
		case CON_IC_STATUS_CONNECTED:
		{
			online_ = true;
			connecting_ = false;
			break;
		}
		case CON_IC_STATUS_DISCONNECTED:
		{
			online_ =  false;
			connecting_ = false;
			/*
			ConIcConnectionError error = 
				con_ic_connection_event_get_error( event );

            switch( error ) {

                case CON_IC_CONNECTION_ERROR_NONE:
                    std::cout << "Libconic thinks there was nothing wrong." << std::endl;
                    break;

                case CON_IC_CONNECTION_ERROR_INVALID_IAP:
                    std::cout << "Invalid (non-existing?) IAP was requested." << std::endl;
                    break;

                case CON_IC_CONNECTION_ERROR_CONNECTION_FAILED:
                    std::cout << "Connection just failed." << std::endl;
                    break;

                case CON_IC_CONNECTION_ERROR_USER_CANCELED:
                    std::cout << "User canceled the connection attempt" << std::endl;
                    break;
            }*/
            
			break;
		}
		case CON_IC_STATUS_DISCONNECTING:
		{
			online_ = false;
			connecting_ = false;
			break;
		}
		default:
		{
			connecting_ = false;
			break;
		}
	}
	
	return;
}


//////////////////////////////////////
//				PRIVATE				//
//////////////////////////////////////


// Initializes engine according to connection profile
bool Connection::initEngine( Profile& profile )
{
	// clear old connections
	clearEngine();
	
	// create server according to the connection profile
	if( profile.wlan )
	{
		// create server for IP connections
		ipserver_ = new Ipengine( profile.ip, profile.port );
		// use ipserver as default server
		engine_ = ipserver_;
	}
	else if( profile.bluetooth )
	{
		// create server for BT connections
		btserver_ = new Btengine( profile.bda, profile.port );
		// use btserver as default server
		engine_ = btserver_;	
	}
	else
	{
		std::cout << "Could not resolve connection profile" << std::endl;
		return false;
	}
	
	
	return true;
}

// Clear previous engine
void Connection::clearEngine()
{
	// disconnect from server
	if( engine_ != NULL && engine_->isConnected() )
	{
		engine_->sendMsg( "06quit" );
		engine_->disconnect();
		engine_ = 0;
	}
	
	if( ipserver_ != NULL )
	{
		delete ipserver_;
		ipserver_ = 0;
	}
	
	if( btserver_ != NULL )
	{
		delete btserver_;
		btserver_ = 0;
	}
}

// Request WLAN connection
bool Connection::openWlanConnection()
{
	if( online_ )
	{
		return true;
	}
	
	// create new ConIcConnection
	ConIcConnection* icConnection = con_ic_connection_new();
	
	if( icConnection == NULL )
	{
		std::cout << "Error creating ConIcConnection" << std::endl;
	}
	
	// connect callback for connection-event
	g_signal_connect( G_OBJECT( icConnection ), "connection-event", 
		G_CALLBACK( Routines::ipConnectionEvent ), this );
	
	// try to connect
	connecting_ = true;
	
	if( !con_ic_connection_connect( icConnection, CON_IC_CONNECT_FLAG_NONE ) )
	{
		std::cout << "Error sending ic connect message to dbus" << std::endl;
	}
	
	// block until signal is received
	while( connecting_ )
	{
		if( g_main_context_pending( NULL ) )
		{
			g_main_context_iteration( NULL, false );
		}
	}
	
	// connection not needed anymore
	g_object_unref( icConnection );
	
	return online_;
}
