/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// file connection.hh
/// \brief Declaration of Connection class

#ifndef CONNECTION_HH
#define CONNECTION_HH

#include "engine.hh"
#include "btengine.hh"
#include "ipengine.hh"
#include "settings.hh"

#include <cstdlib>
#include <string>
#include <conicevent.h>
#include <coniciap.h>
#include <conicconnection.h>
#include <conicconnectionevent.h>

/// Connection status
enum Status
{
	WASCONNECTED, CONNECTED, DISCONNECTED, FAILED, UNKNOWN
};

/// \brief Class to control connection to mythetomerserver
///
/// This class is used to control connecting, disconnecting and sending
/// commands to mythetomerserver. It takes care of opening WLAN connection
/// when necessary. The class utilizes Engine class in underlying details.
class Connection {
	public:
		/// \brief Constructor
		///
		/// Parameters: settings - pointer to application settings object
		Connection( Settings* settings );
		
		/// \brief Destructor
		~Connection();
		
		/// \brief Establish a connection to mythetomerserver
		///
		/// Use this function to establish a network connection to 
		/// mythetomerserver. This function opens automatically WLAN 
		/// connection when needed. The connection is established to the
		/// currently active/selected connection profile.
		///
		/// Return value: CONNECTED - connection was successfully established
		///							  and is ready to be used
		///				  FAILED - failed to establish connection
		Status openConnection();
		
		/// \brief Close the network connection
		///
		/// Use this function to disconnect to mythetomerserver. The connection
		/// is terminated and cannot be used without reconnecting.
		void closeConnection();
		
		/// \brief Change connection profile
		///
		/// Use this function to change connection profile. If connection is
		/// online while function is called, the old connection is terminated
		/// and a new connection is established to the new profile.
		///
		/// Return value: true - connection profile change was successfull
		///				  false - failed to change connection profile
		///
		/// Parameters: name - name of the new connection profile
		bool changeConnectionProfile( std::string const& name );
		
		/// \brief Get info whether the connection is currently established
		///
		/// Use this function the retrieve information whether the connection
		/// is currently established to mythetomerserver.
		///
		/// Return value: true - connection is open
		///				  false - connection is closed
		bool isConnected() const;
		
		/// \brief Send message to server
		///
		/// This function is used to send command or message to active
		/// engine, and thus to the mythetomer server.
		///
		/// Return value: 
		///
		/// Parameters: msg - the message to be sent to mythetomerserver
		bool sendMsg( std::string msg );
		
		/// \brief Handle dbus event from ConIcConnection
		///
		/// This function is used as dbus event handler concerning events
		/// from ConIcConnection. This function updates the online (WLAN)
		/// status of the device to the application.
		///
		/// Return value: none
		///
		/// Parameters: event - event that happened
		void handleConnectionEvent( ConIcConnectionEvent* event );
		
	private:
		
		/// \brief Initializes engine according to connection profile
		///
		/// Use this function to initialize correct engine according to
		/// connection profile. This function must be called before connection
		/// is used.
		///
		/// Return value: true - engine init was successfull
		///				  false - failed to init engine
		bool initEngine( Profile& profile );
		
		/// \brief Clear previous engine
		///
		/// Use this function to clear and remove previous engine. Function
		/// takes care of disconnecting and releasing the previous engine.
		/// This function must be called before any new engine is initialized.
		///
		/// Return value: none
		void clearEngine();
		
		/// \brief Request WLAN connection
		///
		/// Use this function to request WLAN connection. Connection is opened
		/// according to user preferences and using platform tools.
		///
		/// Return value: true - WLAN connection online
		///				  false - WLAN connection offline
		bool openWlanConnection();
		
		/// pointer to currently active engine
		Engine* engine_;
		
		/// pointer to bluetooth server
		Btengine* btserver_;
		
		/// pointer to IP server
		Ipengine* ipserver_;
		
		/// pointer to application settings object
		Settings* settings_;
		
		/// profile currectly in use
		Profile profile_;
		
		/// status of connecting process
		bool connecting_;
		
		/// whether device is online
		bool online_;
};

#endif
