/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// file engine.hh
/// \brief Declaration of Engine class

#ifndef ENGINE_HH
#define ENGINE_HH

#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <cstdlib>
#include <string>

/// \brief Class to model connection engine
///
/// This class is abstract class providing application programming 
/// interface to control the connection to mythetomerserver, to retrieve the
/// status of the connection and to send messages to the server.
class Engine {
	public:
		/// \brief Constructor
		Engine();
		
		/// \brief Virtual destructor
		virtual ~Engine();
		
		/// \brief Connect to remote server, pure virtual function
		///
		/// Use this function to establish a connection to the currently 
		/// selected server. Connection method depends on the implementing
		/// object but may be, for instance, via bluetooth or WLAN.
		///
		/// Return value: true - connection successfully established
		///				  false - failed to establish connection
		virtual bool connectServer() = 0;
		
		/// \brief Disconnect from remote server
		///
		/// Use this function to disconnect from the currently selected server.
		/// After using this function, the connection needs to be established
		/// again in order to use it.
		void disconnect();
		
		/// \brief Get status whether the connection is currently established
		///
		/// Use this function to get the server connection status.
		///
		/// Return value: true - connection to the server is open
		///				  false - connection to the server is closed
		bool isConnected() const;
		
		/// \brief Send message to remote server
		///
		/// Use this function to send message or command to the currently
		/// selected server. Notice that the return value does not implicate
		/// any response to the sent messege.
		///
		/// Return value: true - message was delivered successfully
		///				  failed - message was not delivered, connection 
		///				  		   propably disconnected
		bool sendMsg( std::string message );
		
	protected:
		
		/// \brief Set connection socket
		///
		/// Use this function to set connection socket to parent object in
		/// order to successfully close the socket later on.
		///
		/// Return value: none
		///
		/// Parameters: connection - the socket for established connection 
		void setConnection( int connection );
		
		/// \brief Set connection status
		///
		/// Use this function to set connection status to parent object in
		/// order to successfully retrieve the status from parent.
		///
		/// Return value: none
		///
		/// Parameters: status - connected( true ) / disconnected ( false )
		void setConnected( bool status );
		
	private:
		
		/// the connection socket
		int connection_;
		
		/// the status of connection
		bool connected_;
};

#endif
