/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
/// \file settings.cc
/// \brief Implementation of Settings class

#include "settings.hh"
#include "routines.hh"

#include <iostream>
#include <iterator>

// GCONF general consts
std::string const GCONF_PREFIX = "/apps/maemo/mythetomer";
std::string const GCONF_FULLSCREEN = GCONF_PREFIX + "/fullscreen";
std::string const GCONF_AUTOCONNECT = GCONF_PREFIX + "/autoconnect";
std::string const GCONF_USE_DEFAULT_PROFILE = GCONF_PREFIX + "/usedefaultprofile";
std::string const GCONF_DEFAULT_PROFILE = GCONF_PREFIX + "/defaultprofile";
std::string const GCONF_ACTIVE_PROFILE = GCONF_PREFIX + "/activeprofile";
std::string const GCONF_PROFILE_COUNT = GCONF_PREFIX + "/profilecount";
// GCONF profile consts
std::string const GCONF_PROFILE_PREFIX = GCONF_PREFIX + "/profiles/";
std::string const GCONF_PROFILE_NAME = "/name";
std::string const GCONF_PROFILE_WLAN = "/wlan";
std::string const GCONF_PROFILE_IP = "/ip";
std::string const GCONF_PROFILE_BT = "/bt";
std::string const GCONF_PROFILE_BDA = "/bda";
std::string const GCONF_PROFILE_PORT = "/port";
// GCONF system command consts
std::string const GCONF_COMMAND_PREFIX = GCONF_PREFIX + "/commands/";
std::string const GCONF_COMMAND_NAME = "/name";


//////////////////////////////////////
//				PUBLIC				//
//////////////////////////////////////


// Constructor
Settings::Settings() : fullscreen_( false ), autoconnect_( false ), 
	useDefaultProfile_( true ), defaultProfile_( 0 ), activeProfile_( 0 ), 
	settingsChanged_( false ), gconf_( NULL )
	
{
	// initialize GCONF
	// init type system
	g_type_init();
	
	// load default GConf path
	gconf_ = gconf_client_get_default();
	
	for( unsigned int i = 0; i < 10; ++i )
	{	
		systemCommands_[ i ] = ""; 
	}
	
}

// Destructor
Settings::~Settings()
{
	#ifdef DEGUB
		std::cout << "@Setting destructor" << std::endl;
	#endif
	
	if( settingsChanged_ )
	{
		saveSettings();
	}
	
	// release mem and clear all profiles
	for( unsigned int i = 0; i < profiles_.size(); ++i )
	{
		Profile* profile = profiles_.at( i );
		delete profile;
		profile = 0;
	}
	profiles_.clear();
	
	if( gconf_ != NULL )
	{
		// free gconf settings object
		g_object_unref( gconf_ );
	}
}

// Load application settings
bool Settings::loadSettings()
{
	// make sure that gconf is connected
	if( gconf_ == NULL )
	{
		// init type system
		g_type_init();
		
		// load default GConf path
		gconf_ = gconf_client_get_default();
		
		if( gconf_ == NULL )
		{
			return false;
		}
	}
	
	// get general information
	
	// fullscreen info
	fullscreen_ = gconf_client_get_bool( gconf_, 
		GCONF_FULLSCREEN.c_str(), NULL );
	// autoconnect info
	autoconnect_ = gconf_client_get_bool( gconf_,
		GCONF_AUTOCONNECT.c_str(), NULL );
	// which profile to use
	useDefaultProfile_ = gconf_client_get_bool( gconf_, 
		GCONF_USE_DEFAULT_PROFILE.c_str(), NULL );
	// index of the default profile
	defaultProfile_ = gconf_client_get_int( gconf_,
		GCONF_DEFAULT_PROFILE.c_str(), NULL );
	
	// last used profile
	activeProfile_ = gconf_client_get_int( gconf_, 
		GCONF_ACTIVE_PROFILE.c_str(), NULL );
		
	if( useDefaultProfile_ )
	{
		activeProfile_ = defaultProfile_;
	}
	
	// load connection profiles
	if( !loadProfiles() )
	{
		std::cout << "Error loading profiles" << std::endl;
	}
	
	// load system commands
	if( !loadCommands() )
	{
		std::cout << "Error loading system commands" << std::endl;
	}
	
	return true;
}

// Save application settings
bool Settings::saveSettings()
{	
	if( gconf_ == NULL )
	{
		return false;
	}
	
	// settings has not changed
	if( !settingsChanged_ )
	{
		// return successfully saved
		return true;
	}
	
	// save general information (from general tab)
	
	// fullscreen info
	gconf_client_set_bool( gconf_, GCONF_FULLSCREEN.c_str(), 
		fullscreen_, NULL );
	// autoconnect info
	gconf_client_set_bool( gconf_, GCONF_AUTOCONNECT.c_str(), 
		autoconnect_, NULL );
	// which profile to use
	gconf_client_set_bool( gconf_, GCONF_USE_DEFAULT_PROFILE.c_str(), 
		useDefaultProfile_, NULL );
	// index of the default profile
	gconf_client_set_int( gconf_, GCONF_DEFAULT_PROFILE.c_str(), 
		defaultProfile_, NULL );
	// last used profile
	gconf_client_set_int( gconf_, GCONF_ACTIVE_PROFILE.c_str(), 
		activeProfile_, NULL );
	
	if( !saveProfiles() )
	{
		std::cout << "Saving profiles failed" << std::endl;
		return false;
	}
	
	if( !saveCommands() )
	{
		std::cout << "Saving commands failed" << std::endl;
		return false;
	}
	
	
	settingsChanged_ = false;
	return true;
}

// Get status of fullscreen setting
bool Settings::getFullscreenStatus() const
{
	return fullscreen_;
}

// Get status of connecting automatically setting
bool Settings::getAutoconnectStatus() const
{
	return autoconnect_;
}

// Get status of using default profile setting
bool Settings::getUseDefaultProfile() const
{
	return useDefaultProfile_;
}

// Get number of connection profiles stored
unsigned int Settings::getNumOfProfiles() const
{
	return profiles_.size();
}

// Get connection profile by number
bool Settings::getProfile( unsigned int index, Profile& profile ) const
{
	if( index >= profiles_.size() )
	{
		return false;
	}
	
	profile.name = profiles_.at( index )->name;
	profile.wlan = profiles_.at( index )->wlan;
	profile.ip = profiles_.at( index )->ip;
	profile.bluetooth = profiles_.at( index )->bluetooth;
	profile.bda = profiles_.at( index )->bda;
	profile.port = profiles_.at( index )->port;
	
	return true;
}

// Get connection profile by name
bool Settings::getProfile( std::string const& name, Profile& profile ) const
{
	// find the right profile
	unsigned int profileCount = profiles_.size();
	
	for( unsigned int i = 0; i < profileCount; ++i )
	{
		if( profiles_.at( i )->name == name )
		{
			return getProfile( i, profile );
		}
	}
	
	return false;
}

// Get currently active connection profile
bool Settings::getActiveProfile( Profile& profile ) const
{
	return getProfile( activeProfile_, profile );
}

// Get default connection profile
bool Settings::getDefaultProfile( Profile& profile ) const
{
	return getProfile( defaultProfile_, profile );
}

// Get system command name by command id
bool Settings::getCommand( unsigned int id, std::string& name ) const
{
	unsigned int key = id - 80;
	if( key >= 0 && key < 10 )
	{
		if( systemCommands_[ key ] == "" )
		{
			return false;
		}
		
		name = systemCommands_[ key ];
		
	}
	else
	{
		return false;
	}
	
	return true;
}

// Get system command id by command name
bool Settings::getCommand( std::string const& name, unsigned int& id ) const
{
	// find the right command
	for( unsigned int i = 0; i < NUMBER_OF_COMMANDS; ++i )
	{
		if( systemCommands_[ i ] == name )
		{
			id = i + 80;
			return true;
		}
	}
	
	return false;
}

// Set status of fullscreen setting
void Settings::setFullscreenStatus( bool active )
{
	if( fullscreen_ != active )
	{
		fullscreen_ = active;
		
		// save fullscreen info
		if( !gconf_client_set_bool( gconf_, GCONF_FULLSCREEN.c_str(), 
			fullscreen_, NULL ) )
		{
			// saving failed
			settingsChanged_ = true;
		}
		
	}
}

// Set status of connection automatically setting
void Settings::setAutoconnectStatus( bool active )
{
	if( autoconnect_ != active )
	{
		autoconnect_ = active;
		
		// autoconnect info
		if( !gconf_client_set_bool( gconf_, GCONF_AUTOCONNECT.c_str(), 
			autoconnect_, NULL ) )
		{	
			// saving failed
			settingsChanged_ = true;
		}
	}
}

// Set status of using default profile setting
void Settings::setUseDefaultProfile( bool active )
{
	if( useDefaultProfile_ != active )
	{
		useDefaultProfile_ = active;
		
		// which profile to use
		if( !gconf_client_set_bool( gconf_, GCONF_USE_DEFAULT_PROFILE.c_str(), 
			useDefaultProfile_, NULL ) )
		{
			// saving failed
			settingsChanged_ = true;
		}
	}
}

// Set default connection profile by name
bool Settings::setDefaultProfile( std::string const& name )
{
	// search for same profile name
	// and try to find existing profile
	for( unsigned int i = 0; i < profiles_.size(); ++i )
	{
		if( name == profiles_.at( i )->name )
		{
			// existing profile found
			return setDefaultProfile( i );
		}
	}
	
	return false;
}

// Set active connection profile by name
bool Settings::setActiveProfile( std::string const& name )
{
	#ifdef DEBUG 
		std::cout << "@setActiveProfile" << std::endl;
	#endif
	
	// search for same profile name
	// and try to find existing profile
	for( unsigned int i = 0; i < profiles_.size(); ++i )
	{
		if( name == profiles_.at( i )->name )
		{
			// existing profile found
			//std::cout << "assigning active profile: " << i << std::endl;
			activeProfile_ = i;
			
			// save default profile index
			if( !gconf_client_set_int( gconf_, GCONF_ACTIVE_PROFILE.c_str(), 
				activeProfile_, NULL ) )
			{
				// saving failed
				settingsChanged_ = true;
			}
			
			return true;
		}
	}
	
	return false;
}

// Add connection profile to application
bool Settings::addProfile( Profile const& profile )
{
	Profile* prof = new Profile();
	prof->name = profile.name;
	
	// search for same profile name
	
	bool existing = false;
	int index = 0;
	// check if editing existing profile
	for( unsigned int i = 0; i < profiles_.size(); ++i )
	{
		if( prof->name == profiles_.at( i )->name )
		{
			// existing profile found
			// release just created profile and get the old one
			delete prof;
			prof = profiles_.at( i );
			index = i;
			existing = true;
			break;
		}
	}
	
	prof->wlan = profile.wlan;
	prof->ip = profile.ip;
	prof->bluetooth = profile.bluetooth;
	prof->bda = profile.bda;
	prof->port = profile.port;
	
	// if new, save to memory
	if( !existing )
	{
		// add new profile to container
		profiles_.push_back( prof );
		
		std::string id = Routines::int2string( profiles_.size() - 1 );
		if( saveProfile( prof, id ) )
		{
			// saving failed
			settingsChanged_ = true;
		}
	}
	else
	{
		std::string id = Routines::int2string( index );
		if( saveProfile( prof, id ) )
		{
			// saving failed
			settingsChanged_ = true;
		}
	}
	
	return true;
}

// Delete connection profile from application by name
bool Settings::deleteProfile( std::string const& name )
{
	int index = 0;
	bool found = false;
	
	// find profile
	for( unsigned int i = 0; i < profiles_.size(); ++i )
	{
		if( name == profiles_.at( i )->name )
		{
			// profile found
			index = i;
			found = true;
			
			// get the profile and delete it
			Profile* profile = profiles_.at( i );
			delete profile;
			profile = 0;
	
			break;
		}
	}
	
	// did not find existing profile with such name
	if( !found )
	{
		return false;
	}
	
	// remove the item from vector
	std::vector< Profile* >::iterator i = profiles_.begin();
	i = i + index;
	profiles_.erase( i );
		
	// remove the last key from gconf
	std::string profileKey = Routines::int2string( profiles_.size() );
	
	// construct the key with appropriate profile number
	std::string remove( GCONF_PROFILE_PREFIX + profileKey );
	
	// delete the related profile key
	gconf_client_remove_dir( gconf_, remove.c_str(), NULL );
	
	settingsChanged_ = true;
	
	return true;	
}

// Save single system command
bool Settings::saveCommand( std::string const& id, std::string const& name )
{
	unsigned int number = 0;
	
	if( !Routines::string2int( id, number ) )
	{
		return false;
	}
	
	if( number < 80 || number > 89 )
	{
		std::cout << "Wrong command id: " << id << std::endl;
		return false;
	}
	
	number = number - 80;
	
	if( gconf_ == NULL )
	{
		// saving failed
		settingsChanged_ = true;
		return false;
	}
	
	// construct key
	std::string key( GCONF_COMMAND_PREFIX + Routines::int2string( number ) + GCONF_COMMAND_NAME );
	
	// save command name to permanent memory
	gconf_client_set_string( gconf_, key.c_str(), name.c_str(), NULL );
	
	// save command name to internal variables
	systemCommands_[ number ] = name;
	
	return true;
}


//////////////////////////////////////
//				PRIVATE				//
//////////////////////////////////////


// Set default connection profile by number
bool Settings::setDefaultProfile( unsigned int index )
{
	if( index >= profiles_.size() )
	{
		return false;
	}
	else
	{
		defaultProfile_ = index;
		
		// save default profile index
		if( !gconf_client_set_int( gconf_, GCONF_DEFAULT_PROFILE.c_str(), 
			defaultProfile_, NULL ) )
		{
			// saving failed
			settingsChanged_ = true;
		}
	}
	
	return true;
}

// Load all connection profiles from permanent memory (GConf)
bool Settings::loadProfiles()
{
	// get number of profiles
	unsigned int profileCount = gconf_client_get_int( gconf_, 
		GCONF_PROFILE_COUNT.c_str(), NULL );
	
	for( unsigned int i = 0; i < profileCount; ++i )
	{
		
		std::string id = Routines::int2string( i );
		
		if( id == "" )
		{
			break;
		}
		
		Profile* profile = new Profile();
		
		// construct keys with appropriate profile number
		std::string name( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_NAME );
		std::string wlan( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_WLAN );
		std::string ip( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_IP );
		std::string bt( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_BT );
		std::string bda( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_BDA );
		std::string port( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_PORT );
		
		
		// get profile variables
		profile->name = gconf_client_get_string( gconf_, name.c_str(), NULL );
		profile->wlan = gconf_client_get_bool( gconf_, wlan.c_str(), NULL );
		profile->ip = gconf_client_get_string( gconf_, ip.c_str(), NULL );
		profile->bluetooth = gconf_client_get_bool( gconf_, bt.c_str(), NULL );
		profile->bda = gconf_client_get_string( gconf_, bda.c_str(), NULL );
		profile->port = gconf_client_get_int( gconf_, port.c_str(), NULL );
		
		profiles_.push_back( profile );
	}
	
	return true;
	
}

// Save all application connection profiles
bool Settings::saveProfiles()
{
	// save information about profiles
	unsigned int profileCount = profiles_.size();
	
	// save number of profiles
	gconf_client_set_int( gconf_, GCONF_PROFILE_COUNT.c_str(), profileCount, NULL );
	
	for( unsigned int i = 0; i < profileCount; ++i )
	{
		std::string id = Routines::int2string( i );
		
		if( id == "" )
		{
			break;
		}
		
		// save each profiles
		if( !saveProfile( profiles_.at( i ), id ) )
		{
			std::cout << "Error saving profile " << profiles_.at( i )->name
			<< std::endl;
		}
		
	}
	
	return true;
}



// Save single profile to permanent memory (GConf)
bool Settings::saveProfile( Profile* profile, std::string& id )
{
	if( profile == NULL || gconf_ == NULL )
	{
		return false;
	}
	
	// construct keys with appropriate profile number
	std::string name( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_NAME );
	std::string wlan( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_WLAN );
	std::string ip( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_IP );
	std::string bt( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_BT );
	std::string bda( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_BDA );
	std::string port( GCONF_PROFILE_PREFIX + id + GCONF_PROFILE_PORT );
	
	// save profile variables
	gconf_client_set_string( gconf_, name.c_str(), profile->name.c_str(), NULL );
	gconf_client_set_bool( gconf_, wlan.c_str(), profile->wlan, NULL );
	gconf_client_set_string( gconf_, ip.c_str(), profile->ip.c_str(), NULL );
	gconf_client_set_bool( gconf_, bt.c_str(), profile->bluetooth, NULL );
	gconf_client_set_string( gconf_, bda.c_str(), profile->bda.c_str(), NULL );
	gconf_client_set_int( gconf_, port.c_str(), profile->port, NULL );
	
	// save information about profiles
	unsigned int profileCount = profiles_.size();
	
	// save number of profiles
	gconf_client_set_int( gconf_, GCONF_PROFILE_COUNT.c_str(), profileCount, NULL );
	
	return true;
}

// Load all system commands from permanent memory (GConf)
bool Settings::loadCommands()
{
	for( unsigned int i = 0; i < 10; ++i )
	{
		// construct key
		std::string id = Routines::int2string( i );
		std::string name( GCONF_COMMAND_PREFIX + id + GCONF_COMMAND_NAME );
		std::string dir( GCONF_COMMAND_PREFIX + id );
		
		// ensure that directory exist
		if( gconf_client_dir_exists( gconf_, dir.c_str(), NULL ) )
		{
			// load command variables
			std::string value = gconf_client_get_string( gconf_, name.c_str(), NULL );
			systemCommands_[ i ] = value;
		}
	}
	
	return true;
}

// Save all application system commands to permanent memory
bool Settings::saveCommands()
{
	// save system commands
	for( unsigned int i = 0; i < NUMBER_OF_COMMANDS; ++i )
	{
		std::string id = Routines::int2string( i + 80 );
		
		if( id == "" )
		{
			break;
		}
		
		// save each command
		if( !saveCommand( id, systemCommands_[ i ] ) )
		{
			std::cout << "Error saving command " 
			<< systemCommands_[ i ] << std::endl;
		}
	}
	
	return true;
}

