/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file settings.hh
/// \brief Declaration of Settings class

#ifndef SETTINGS_H
#define SETTINGS_H

#include <string>
#include <vector>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

/// data structure to present connection profile information
struct Profile
{
	std::string name;
	bool wlan;
	std::string ip;
	bool bluetooth;
	std::string bda;
	int port;
};

/// number of available system commands
unsigned int const NUMBER_OF_COMMANDS = 10;

/// \brief Class to hold and modify all application settings
///
/// This class holds all application settings. It is used to get and modify
/// those settings as well through getters and setters. Application settings
/// data are stored in class member variables and saved using GConf.
class Settings {
	public:
		/// \brief Constructor
		Settings();
		
		/// \brief Destructor
		~Settings();
		
		/// \brief Load application settings
		///
		/// Use this function to load application settings from permanent
		/// memory into application awareness. This function MUST be called
		/// before any other functions in order to get reasonable
		/// settings data via getters.
		///
		/// Return value: true - settings were loaded successfully
		///				  false - error happened while loading settings
		bool loadSettings();
		
		/// \brief Save application settings
		///
		/// Use this function to save application settings to permanent
		/// memory. This function must be called in order to make sure that
		/// all application settings are saved.
		///
		/// Return value: true - settings were saved successfully
		///				  false - error happened while saving settings
		bool saveSettings();
		
		
		/// \brief Get status of fullscreen setting
		///
		/// Use this function to get information about status of 
		/// wether user wants to start application fullscreen.
		///
		/// Return value: true - fullscreen at start up
		///				  false - non-fullscreen at start up
		bool getFullscreenStatus() const;
		
		/// \brief Get status of connecting automatically setting
		///
		/// Use this function to get information about status of
		/// wether user wants to connect automatically at application
		/// startup.
		///
		/// Return value: true - connect automatically at start up
		///				  false - no automatic connection at start up
		bool getAutoconnectStatus() const;
		
		/// \brief Get status of using default profile setting
		///
		/// Use this function to get information about wether user
		/// wants to use default profile as connection profile.
		///
		/// Return value: true - default profile used
		/// 			  false - profile from last session is used
		bool getUseDefaultProfile() const;
		
		/// \brief Get number of connection profiles stored
		///
		/// Use this function to get information about the amount of
		/// connection profiles available.
		///
		/// Return value: number of connection profiles available
		unsigned int getNumOfProfiles() const;
		
		/// \brief Get connection profile by number
		///
		/// Use this function to get connection profile information by
		/// using the number of connection profile requested.
		///
		/// Return value: true - parameter profile contains valid data
		///				  false - parameter profile does not contain valid data
		///
		/// Parameters: index - number of requested profile
		///				profile - connection profile filled with information
		bool getProfile( unsigned int index, Profile& profile ) const;
		
		/// \brief Get connection profile by name
		///
		/// Use this function to get connection profile information by
		/// using the profile name of connection profile requested.
		///
		/// Return value: true - parameter profile contains valid data
		///				  false - parameter profile does not contain valid data
		///
		/// Parameters: name - name of requested profile
		///				profile - connection profile filled with information
		bool getProfile( std::string const& name, Profile& profile ) const;
		
		/// \brief Get currently active connection profile
		///
		/// Use this function to get the currently active connection profile.
		///
		/// Return value: true - parameter profile contains valid data
		///				  false - parameter profile does not contain valid data
		///
		/// Parameters: profile - connection profile filled with information
		bool getActiveProfile( Profile& profile ) const;
		
		/// \brief Get default connection profile
		///
		/// Use this function to get the connection profile that has been 
		/// saved as default profile
		///
		/// Return value: true - parameter profile contains valid data
		///				  false - parameter profile does not contain valid data
		///
		/// Parameters: profile - connection profile filled with information
		bool getDefaultProfile( Profile& profile ) const;
		
		/// \brief Get system command name by command id
		///
		/// Use this function to get system command name by
		/// using the command of system command requested.
		///
		/// Return value: true - parameter name contains valid data
		///				  false - parameter name does not contain valid data
		///
		/// Parameters: id - command number of the system command
		///				name - the name of the system command requested by id
		bool getCommand( unsigned int id, std::string& name ) const;
		
		/// \brief Get system command id by command name
		///
		/// Use this function to get command of system command by
		/// using the name of system command requested.
		///
		/// Return value: true - parameter id contains valid data
		///				  false - parameter id does not contain valid data
		///
		/// Parameters: name - name of the system command
		///				id - the command number of the system command
		///					 requested by name 
		bool getCommand( std::string const& name, unsigned int& id ) const;
		
		/// \brief Set status of fullscreen setting
		///
		/// Use this function to set status of wether user wants to
		/// start application fullscreen.
		///
		/// Return value: none
		///
		/// Parameters: active - fullscreen (true) / non-fullscreen (false)
		void setFullscreenStatus( bool active );
		
		/// \brief Set status of connection automatically setting
		///
		/// Use this function to set status of wether user wants to
		/// automatically connect as application start up.
		///
		/// Return value: none
		///
		/// Parameters: active - connect automatically (true) / no 
		///						 automatic connection (false)
		void setAutoconnectStatus( bool active );
		
		/// \brief Set status of using default profile setting
		///
		/// Use this function to set status of wether user wants to
		/// use default connection profile as the connection profile.
		///
		/// Return value: none
		///
		/// Parameters: active - default connection profile used (true) / 
		///						 connection profile from last session is 
		///						 used (false)
		void setUseDefaultProfile( bool active );
		
		/// \brief Set default connection profile by name
		///
		/// Use this function to set the default connection profile by using
		/// the connection profile name.
		///
		/// Return value: true - default connection profile set successfully
		///				  false - failed to set default connection profile
		/// Parameters: name - connection profile name to be set as default
		bool setDefaultProfile( std::string const& name );
		
		/// \brief Set active connection profile by name
		///
		/// Use this function to set the active connection profile by using
		/// the connection profile name.
		///
		/// Return value: true - active connection profile set successfully
		///				  false - failed to set active connection profile
		/// Parameters: name - connection profile name to be set as default
		bool setActiveProfile( std::string const& name );
		
		/// \brief Add connection profile to application
		///
		/// Use this function to add new connection profile to the application.
		/// After using this function, profile will be saved and available
		/// in the future sessions, too.
		///
		/// Return value: true - adding of connection profile succeeded
		///				  false - connection profile was not successfully saved
		///
		/// Parameters: profile - the connection profile to be added 
		bool addProfile( Profile const& profile );
		
		/// \brief Delete connection profile from application by name
		///
		/// Use this function to delete existing connection profile by using
		/// using the name of the profile as identifier. After using this
		/// function, profile will be removed and not available in the
		/// future sessions either.
		///
		/// Return value: true - connection profile deleted successfully
		/// 			  false - deletion of connection profile failed
		///
		/// Parameters: name - the name of the connection profile to be removed
		bool deleteProfile( std::string const& name );
		
		/// \brief Save single system command
		///
		/// Use this function to save or update single system command. System
		/// commands are identified using the command id. Function recognizes
		/// wether adding new command or editing existing one.
		///
		/// Return value: true - saving of system command was successfull
		///				  false - system command saving failed
		///
		/// Parameters: id - the command id of the system command to be saved
		///				name - the command name of the system command to be
		///					   saved
		bool saveCommand( std::string const& id, std::string const& name );
		
	private:
		
		/// Private member functions
		
		/// \brief Set default connection profile by number
		///
		/// Use this function to set the default connection profile by using
		/// the connection profile number.
		///
		/// Return value: true - default connection profile set successfully
		///				  false - failed to set default connection profile
		///
		/// Parameters: index - the number of connection profile to be set
		///						as the default connection profile
		bool setDefaultProfile( unsigned int index );
		
		/// \brief Load all connection profiles from permanent memory (GConf)
		///
		/// Use this function to load all application connection profiles
		/// from permanent memory. 
		///
		/// Return value: true - loading succeeded, profiles are available
		///				  false - loading failed, profiles may not be available
		bool loadProfiles();
		
		/// \brief Save all application connection profiles
		///
		/// Use this function to save or update all application connection
		/// profiles to permanent memory. After using this function, profiles
		/// will be saved and available in the future sessions, too.
		///
		/// Return value: true - saving succeeded
		///				  false - failed to save all connection profiles
		bool saveProfiles();
		
		/// \brief Save single profile to permanent memory (GConf)
		///
		/// Use this function to save or update single connection profile to
		/// permanent memory. After using this fuction, the profile will
		/// be saved and available in the future sessions, too.
		///
		/// Return value: true - saving succeed
		///				  false - failed to save the connection profile
		///
		/// Parameters: profile - the connection profile to save
		///				id - the connection profile indentification
		bool saveProfile( Profile* profile, std::string& id );
		
		/// \brief Load all system commands from permanent memory (GConf)
		///
		/// Use this function to load all application system commands from
		/// permanent memory.
		///
		/// Return value: true - loading succeeded, commands are available
		///				  false - loading failed, commands may not be available
		bool loadCommands();
		
		/// \brief Save all application system commands to permanent memory
		///
		/// Use this function to save or update all application system
		/// commands to permanent memory (via GConf). After using this 
		/// function, system commands will be saved and available in the 
		/// future sessions, too.
		///
		/// Return value: true - saving succeeded
		///				  false - failed to save all system commands
		bool saveCommands();
		
		
		/// Private member variables
		
		/// fullscreen status
		bool fullscreen_;
		
		/// autoconnection status
		bool autoconnect_;
		
		/// wether to use default connection profile or not
		bool useDefaultProfile_;
		
		/// the default connection profile
		unsigned int defaultProfile_;
		
		/// the active connection profile
		unsigned int activeProfile_;
		
		/// wether settings have changed and they are not save yet
		bool settingsChanged_;
		
		/// pointer to GConfClient
		GConfClient* gconf_;
		
		/// data structure to hold all connection profiles
		std::vector< Profile* > profiles_;
		
		/// data structure to hold system commands
		std::string systemCommands_[NUMBER_OF_COMMANDS];
		
};

#endif
