/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file ui.hh
/// \brief Declaration of UI class

#ifndef UI_H
#define UI_H

#include "settings.hh"
#include "connection.hh"

#include <string>
#include <vector>
#include <libosso.h>
#include <hildonmm/program.h>
#include <hildonmm/window.h>
#include <hildonmm/note.h>
#include <gtkmm/stock.h>
#include <gtkmm/box.h>
#include <gtkmm/button.h>
#include <gtkmm/image.h>
#include <gtkmm/label.h>
#include <gtkmm/notebook.h>
#include <gtkmm/comboboxtext.h>
#include <gtkmm/entry.h>
#include <gtkmm/table.h>
#include <gtkmm/frame.h>
#include <gtkmm/menu.h>

/// Data structure to present user created system commands
struct Command
{
	/// name of the command
	std::string name;
	/// command string
	std::string command;
	/// pointer to menu item
	Gtk::MenuItem* menuItem;
};

/// \brief Class to present the main User Interface
///
/// This class presents the main User Interface and menu structure to the user.
/// All user actions happen through this class, as it is everything that is
/// viewable by the user.
class Ui : public Hildon::Window {
	public:
		/// \brief Constructor
		///
		/// Parameters: ossoContext - pointer to osso instance
		///				dbus - pointer to system dbus
		///				settings - pointer to initiated settings class
		///				connection - pointer to initiated connection class
		Ui( osso_context_t* ossoContext, DBusConnection* dbus, 
			Settings* settings, Connection* connection );
		
		/// \brief Destructor
		virtual ~Ui();
		
		/// \brief Show information banner to user
		///
		/// Use this function to show infoprint (information banner) on 
		/// screen statusbar.
		///
		/// Return value: none
		///
		/// Parameters: message - the text to be shown
		void showMessage( std::string const& message );
		
		/// \brief Callback function to open connection
		///
		/// Use this function to establish a connection to the active 
		/// connection profile. Required for automatic connection feature.
		///
		/// Return value: none
		void openConnection();
		
		
		/// \brief Callback function for inserting data after bluetooth search
		///
		/// Use this function only to insert Bluetooth device name and device
		/// address to search dialog.
		///
		/// Return value: none
		///
		/// Parameters: name - bluetooth device name
		///				bda - bluetooth device address
		void setDevice( std::string const& name, std::string const& bda );
		
	private:
		
		/// \brief Send user initiated command to the server
		///
		/// Use this function to send user initiated command to the server.
		/// This function can be used as callback function for e.g. button.
		/// If the message cannot be delivered, question is asked wheather
		/// the user wants to establish connection.
		///
		/// Return value: none
		///
		/// Parameters: message - the command to be sent to the server
		void userAction( std::string const& message );
		
		/// \brief Show dialog to send custom command
		///
		/// Use this function to show user a dialog to send custom command
		/// to server.
		///
		/// Return value: none
		void showCustomCommandDialog();
		
		/// \brief Show dialog to edit application settings
		///
		/// Use this function to show user a dialog to view and edit
		/// application settings.
		///
		/// Return value: none
		void showSettingsDialog();
		
		/// \brief (Re-)build profiles combo box in Settings dialog
		///
		/// Use this function to re-fill profiles combo box in Settings
		/// dialog. Items in the combo box are updated only after calling
		/// this function.
		///
		/// Return value: none
		void buildProfilesCombo();
		
		/// \brief (Re-)build system commands combo box in Settings dialog
		///
		/// Use this function to re-fill system commands combo box in Settings
		/// dialog. Items in the combo box are updated only after calling
		/// this function.
		///
		/// Return value: none
		void buildSystemCommandsCombo();
		
		/// \brief Show dialog to create new Connection Profile
		///
		/// Use this function to show dialog to create a new Connection
		/// Profile.
		///
		/// Return value: none
		void showNewProfileDialog();
		
		/// \brief Show dialog to edit old Connection Profile
		///
		/// Use this function to show dialog to edit old Connection
		/// Profile.
		///
		/// Return value: none
		void showEditProfileDialog();
		
		/// \brief Show dialog to edit Connection Profile
		///
		/// Use this function to show and edit Connection Profile.
		///
		/// Return value: none
		///
		/// Parameters: profile - pointer to profile under consideration
		void showProfileDialog( Profile* profile );
		
		
		
		
		/// \brief Create a new Connection Profile to the application
		///
		/// Use this function to add a new Connection Profile to the
		/// application. This function validates the profile and saves it
		/// for future use.
		///
		/// Return value: true - profile creation succeeded
		///				  false - failed to create profile
		///
		/// Parameters: name - profile name
		///				isWlan - is profile WLAN profile
		///				address - ip address or bluetooth device address
		///				port - server port number
		bool createProfile( std::string name, bool isWlan, std::string address,
			int port = 0 );
		
		/// \brief Callback function for connection profile delete button
		///
		/// Use this function as callback function for deleting selected
		/// profile in settings dialog profile tab.
		///
		/// Return value: none
		void deleteProfile();
		
		/// \brief Callback function for connection profile set default button
		///
		/// Use this function as callback function for setting default
		/// profile in settings dialog profile tab.
		///
		/// Return value: none
		void setDefaultProfile();
		
		
		
		/// \brief Show a dialog to edit system command menu titles
		///
		/// Use this function to show user a dialog to edit system command
		/// menu titles.
		///
		/// Return value: none
		void showSystemCommandDialog();
		
		/// \brief Update system command information
		///
		/// Use this function when system command information changes. This
		/// function updates application settings and menu items.
		///
		/// Return value: true - updating was successfull
		///				  false - updating failed
		///
		/// Parameters: name - system command menu item name
		///				id - system command id number
		bool updateSystemCommand( std::string const& name, std::string const& id );
		
		/// \brief Show application help to user
		///
		/// Use this function to open application help.
		///
		/// Return value: none
		void showHelp();
		
		/// \brief Show application about dialog
		///
		/// Use this function to show about dialog.
		///
		/// Return value: none
		void showAbout();
		
		/// \brief Change application fullscreen status
		///
		/// Use this function to change application view to fullscreen/normal
		/// view. Calling this function toggles the option.
		///
		/// Return value: none
		void toggleFullscreen();
		
		/// \brief Callback function for hardware key presses
		///
		/// Use this function as callback function for hardware key presses.
		///
		/// Return value: true - the event was handled
		///				  false - the event was not handled in this function
		///
		/// Parameters: event - the GDK key event that occurred
		bool hwKeyPressed( GdkEventKey* event );
		
		
		/// \brief Construct application main menu
		///
		/// Use this function to construct application main menu.
		///
		/// Return value: none
		void buildMenu();
		
		/// \brief Construct System commands -submenu
		///
		/// Use this function to construct System commands -submenu.
		///
		/// Return value: none
		void buildSystemCommandsSubmenu();
		
		/// \brief Remove system command from application
		///
		/// Use this function to remove system command from application. This
		/// function updates combo box and settings respectively.
		///
		/// Return value: none
		void removeSystemCommand();
		
		/// \brief Clear all system command menu items
		///
		/// Use this function to release dynamically acquired memory for
		/// system commands menu items. 
		///
		/// Return value: none
		void clearSystemCommands();
		
		/// \brief Construct Connection profile -submenu
		///
		/// Use this function to construc t Connection profile -submenu.
		///
		/// Return value: none
		void buildSelectProfileSubmenu();
		
		/// \brief Clear all Connection profile menu items
		///
		/// Use this function release dynamically acquired memory for
		/// connection profile menu items.
		///
		/// Return value: none
		void clearSelectProfileMenuItems();
		
		/// \brief Callback function for changing active connection
		///
		/// Use this function to change active connection to server.
		///
		/// Return value: none
		///
		/// Parameters: name - name of the new connection profile
		void changeConnection( std::string const& name );
		
		/// \brief Construct the main User Interface
		///
		/// Use this function to construct the main User Interface including
		/// all buttons and functions.
		///
		/// Return value: none
		void buildInterface();
		
		/// \brief Start searching Bluetooth devices
		///
		/// Use this function to start seaching Bluetooth devices in a
		/// neighbourhood.
		///
		/// Return value: none
		void searchBtDevices();
		
		
		/// Private member variables
		
		/// pointer to osso instance
		osso_context_t* osso_;
		
		/// pointer to system dbus
		DBusConnection* dbus_;
		
		/// pointer to application settings class
		Settings* settings_;
		
		/// pointer to connection class
		Connection* connection_;
		
		/// wheather the view is fullscreen or not
		bool fullscreen_;
		
		/// wheather submenu building is ongoing
		bool buildingSubmenu_;
		
		// variables related to the main screen UI
		
		Gtk::HBox hBox_; 			// the outer container
		
		Gtk::VBox vBoxLeft_; 		// the left most column
		Gtk::Button btnUp_; 		// button up
		Gtk::Image imgUp_;			// image up
		Gtk::Button btnEnter_; 		// button enter
		Gtk::Image imgEnter_;		// image enter
		Gtk::Button btnDown_; 		// button down
		Gtk::Image imgDown_;		// image down
		
		Gtk::VBox vBoxCenter_; 		// center column
		
		// first row
		Gtk::HBox hBox1_; 			// container for first row
		Gtk::Button btnVolUp_; 		// button volume up
		Gtk::Image imgVolUp_;		// image volume up
		Gtk::Button btnOne_; 		// button 1
		Gtk::Button btnTwo_; 		// button 2
		Gtk::Button btnThree_; 		// button 3
		Gtk::Button btnChUp_; 		// button channel up
		Gtk::Image imgChUp_;		// image channel up
		
		// second row
		Gtk::HBox hBox2_;			// container for second row
		Gtk::Label lblVol_;			// label for volume
		Gtk::Button btnFour_; 		// button 4
		Gtk::Button btnFive_; 		// button 5
		Gtk::Button btnSix_; 		// button 6
		Gtk::Label lblCh_;			// label for channel
		
		// third row
		Gtk::HBox hBox3_;			// container for third row
		Gtk::Button btnVolDown_;	// button volume down
		Gtk::Image imgVolDown_;		// image volume down
		Gtk::Button btnSeven_; 		// button 7
		Gtk::Button btnEight_; 		// button 8
		Gtk::Button btnNine_; 		// button 9
		Gtk::Button btnChDown_; 	// button channel down
		Gtk::Image imgChDown_;		// image channel down
		
		// fourth row
		Gtk::HBox hBox4_;			// container for fourth row
		Gtk::Button btnMenu_; 		// button menu
		Gtk::Label lblEmpty1_; 		// empty space
		Gtk::Button btnZero_; 		// button 0
		Gtk::Label lblEmpty2_; 		// empty space
		Gtk::Button btnEsc_; 		// button escape
		
		// fifth row
		Gtk::HBox hBox5_;			// container for fifth row
		Gtk::Button btnRewind_; 	// button rewind
		Gtk::Image imgRewind_;		// image rewind
		Gtk::Button btnPause_; 		// button pause
		Gtk::Image imgPause_;		// image pause
		Gtk::Button btnForward_;	// button forward
		Gtk::Image imgForward_;		// image forward
		
		// sixth row
		Gtk::HBox hBox6_;			// container for sixth row
		Gtk::Button btnPrevious_;	// button previous
		Gtk::Image imgPrevious_;	// image previous
		Gtk::Button btnRecord_;		// button record
		Gtk::Image imgRecord_;		// image record
		Gtk::Button btnNext_; 		// button next
		Gtk::Image imgNext_;		// image next
		
		Gtk::VBox vBoxRight_; 		// the right most column
		Gtk::Button btnRight_; 		// button right
		Gtk::Image imgRight_;		// image right
		Gtk::Button btnEnterRight_;	// button enter
		Gtk::Image imgEnterRight_;	// image enter
		Gtk::Button btnLeft_; 		// button left
		Gtk::Image imgLeft_;		// image left
		
		// variables related to settings dialog
		
		// notebook(tabs) for settings dialog
		Gtk::Notebook ntbkSettings_;
		
		// combobox showing connection profile names
		Gtk::ComboBoxText cmbProfiles_;
		
		// combobox showing system command names
		Gtk::ComboBoxText cmbSystemCommands_;
		
		// data structure holding system commands information
		std::vector< Command* > systemCommands_;
		
		// data structure holding connection profiles menu items
		std::vector< Gtk::MenuItem* > profileMenuItems_;
		
		// text entry for Bluetooth Device Address
		Gtk::Entry* ntrBda_;
		
		// variables related to menus
		
		Gtk::Menu menu_;					// main menu
		
		Gtk::Menu gotoSubmenu_;				// Go to -submenu
		Gtk::MenuItem itemGoto_;			// Go to -submenu item
		Gtk::MenuItem itemMainMenu_;		// Main menu -item
		Gtk::MenuItem itemProgramGuide_;	// Program Guide -item
		Gtk::MenuItem itemLiveTV_;			// Live TV -item
		Gtk::MenuItem itemTVRecordings_;	// TV Recordings -item
		Gtk::MenuItem itemMythVideo_;		// MythVideo -item
		Gtk::MenuItem itemPlayDVD_;			// Play DVD -item
		Gtk::MenuItem itemPlayMusic_;		// Play music -item
		
		Gtk::Menu extraCommandsSubmenu_;	// Extra commmands -submenu
		Gtk::MenuItem itemExtraCommands_;	// Extra commands -submenu item
		Gtk::MenuItem itemAspectRatio_;		// Aspect ratio -item
		Gtk::MenuItem itemTimeStretch_;		// Time stretch -item
		Gtk::MenuItem itemInputCard_;		// Input card -item
		Gtk::MenuItem itemSleepTimer_;		// Sleep timer -item
		Gtk::SeparatorMenuItem itemSep0_;	// Separator item
		Gtk::MenuItem itemCustomCommand_;	// Custom command -item
		
		Gtk::Menu* systemCommandsSubmenu_;	// System commands -submenu
		Gtk::MenuItem itemSystemCommands_;	// System commands -submenu item
		
		Gtk::SeparatorMenuItem itemSep1_;	// Separator item
		
		Gtk::Menu* selectProfileSubmenu_;	// Connection profile -submenu
		Gtk::MenuItem itemSelectProfile_;	// Connection profile -submenu item
		
		Gtk::SeparatorMenuItem itemSep4_;	// Separator item
		
		Gtk::MenuItem itemOpenConnection_;	// Open connection -item
		Gtk::MenuItem itemCloseConnection_;	// Close connection -item
		
		Gtk::SeparatorMenuItem itemSep2_;	// Separator item
		
		Gtk::Menu toolsSubmenu_;			// Tools -submenu
		Gtk::MenuItem itemTools_;			// Tools -submenu item
		Gtk::MenuItem itemSettings_;		// Settings -item
		Gtk::MenuItem itemHelp_;			// Help -item
		Gtk::MenuItem itemAbout_;			// About -item
		
		Gtk::SeparatorMenuItem itemSep3_;	// Separator item
		
		Gtk::MenuItem itemClose_;			// Close item
		
};

#endif
